@@ -302,26 +302,15 @@ add_glyph (ThaiFontInfo     *font_info,
   pango_font_get_glyph_extents (font_info->font,
 				glyphs->glyphs[index].glyph, &ink_rect, &logical_rect);
 
-  if (combining)
+  if (combining || logical_rect.width > 0)
     {
-      glyphs->glyphs[index].geometry.x_offset =
-	glyphs->glyphs[index - 1].geometry.width;
-      glyphs->glyphs[index].geometry.width =
-	logical_rect.width + glyphs->glyphs[index - 1].geometry.width;
-      glyphs->glyphs[index - 1].geometry.width = 0;
+      glyphs->glyphs[index].geometry.x_offset = 0;
+      glyphs->glyphs[index].geometry.width = logical_rect.width;
     }
   else
     {
-      if (logical_rect.width > 0)
-        {
-	 glyphs->glyphs[index].geometry.x_offset = 0;
-	 glyphs->glyphs[index].geometry.width = logical_rect.width;
-	}
-      else
-        {
-	 glyphs->glyphs[index].geometry.x_offset = ink_rect.width;
-	 glyphs->glyphs[index].geometry.width = ink_rect.width;
-        }
+      glyphs->glyphs[index].geometry.x_offset = ink_rect.width;
+      glyphs->glyphs[index].geometry.width = ink_rect.width;
     }
   glyphs->glyphs[index].geometry.y_offset = 0;
 }
