@@ -95,13 +95,12 @@ void
 pango_font_description_set_family (PangoFontDescription *desc,
 				   const char           *family)
 {
+  gchar *old_family = NULL;
+	
   g_return_if_fail (desc != NULL);
 
-  if (desc->family_name == family)
-    return;
-  
   if (desc->family_name && !desc->static_family)
-    g_free (desc->family_name);
+    old_family = desc->family_name;
 
   if (family)
     {
@@ -114,6 +113,9 @@ pango_font_description_set_family (PangoFontDescription *desc,
       desc->family_name = NULL;
       desc->mask &= ~PANGO_FONT_MASK_FAMILY;
     }
+
+  if (old_family)
+    g_free (old_family);
 }
 
 /**
@@ -616,6 +618,24 @@ pango_font_description_equal (const PangoFontDescription  *desc1,
 	   (desc1->family_name && desc2->family_name && g_ascii_strcasecmp (desc1->family_name, desc2->family_name) == 0)));
 }
 
+#define TOLOWER(c) \
+  (((c) >= 'A' && (c) <= 'Z') ? (c) - 'A' + 'a' : (c))
+
+static guint
+case_insensitive_hash (const char *key)
+{
+  const char *p = key;
+  guint h = TOLOWER (*p);
+
+  if (h)
+    {
+      for (p += 1; *p != '\0'; p++)
+	h = (h << 5) - h + TOLOWER (*p);
+    }
+
+  return h;
+}
+
 /**
  * pango_font_description_hash:
  * @desc: a #PangoFontDescription
@@ -633,7 +653,7 @@ pango_font_description_hash (const PangoFontDescription *desc)
   hash = desc->mask;
   
   if (desc->mask & PANGO_FONT_MASK_FAMILY)
-    hash = g_str_hash (desc->family_name);
+    hash = case_insensitive_hash (desc->family_name);
   if (desc->mask & PANGO_FONT_MASK_SIZE)
     hash ^= desc->size;
   if (desc->mask & PANGO_FONT_MASK_STYLE)
@@ -1341,6 +1361,36 @@ pango_font_family_list_faces (PangoFontFamily  *family,
   PANGO_FONT_FAMILY_GET_CLASS (family)->list_faces (family, faces, n_faces);
 }
 
+/**
+ * pango_font_family_is_monospace:
+ * @family: a #PangoFontFamily
+ *
+ * A monospace font is a font designed for text display where the the
+ * characters form a regular grid. For Western languages this would
+ * mean that the advance width of all characters are the same, but
+ * this categorization also includes Asian fonts which include
+ * double-width characters: characters that occupy two grid cells.
+ * g_unichar_iswide() returns a result that indicates whether a
+ * character is typically double-width in a monospace font.
+ * 
+ * The best way to find out the grid-cell size is to call
+ * pango_font_metrics_get_approximate_digit_width(), since the results
+ * of pango_font_metrics_get_approximate_char_width() may be affected
+ * by double-width characters.  
+ * 
+ * Return value: TRUE if the family is monospace.
+ **/
+gboolean 
+pango_font_family_is_monospace (PangoFontFamily  *family)
+{
+  g_return_val_if_fail (PANGO_IS_FONT_FAMILY (family), FALSE);
+
+  if (PANGO_FONT_FAMILY_GET_CLASS (family)->is_monospace)
+    return PANGO_FONT_FAMILY_GET_CLASS (family)->is_monospace (family);
+  else
+    return FALSE;
+}
+
 /*
  * PangoFontFace
  */
@@ -1412,3 +1462,37 @@ pango_font_face_get_face_name (PangoFontFace *face)
 
   return PANGO_FONT_FACE_GET_CLASS (face)->get_face_name (face);
 }
+
+/**
+ * pango_font_face_list_sizes:
+ * @face: a #PangoFontFace.
+ * @sizes: location to store a pointer to an array of int. This array
+ *         should be freed with g_free().
+ * @n_sizes: location to store the number of elements in @sizes
+ * 
+ * List the available sizes for a font. This is only applicable to bitmap
+ * fonts. For scalable fonts, stores %NULL at the location pointed to by
+ * @sizes and 0 at the location pointed to by @n_sizes. The sizes returned
+ * are in pango units and are sorted in ascending order.
+ *
+ * Since: 1.4
+ **/
+void
+pango_font_face_list_sizes (PangoFontFace  *face, 
+                            int           **sizes, 
+                            int            *n_sizes)
+{
+  g_return_if_fail (PANGO_IS_FONT_FACE (face));
+  g_return_if_fail (sizes == NULL || n_sizes != NULL);
+
+  if (n_sizes == NULL)
+    return;
+
+  if (PANGO_FONT_FACE_GET_CLASS (face)->list_sizes != NULL)
+    PANGO_FONT_FACE_GET_CLASS (face)->list_sizes (face, sizes, n_sizes);
+  else
+    {
+      *sizes = NULL;
+      *n_sizes = 0;
+    }
+}
