@@ -83,7 +83,7 @@ pango_context_get_type (void)
 static void
 pango_context_init (PangoContext *context)
 {
-  context->base_dir = PANGO_DIRECTION_LTR;
+  context->base_dir = PANGO_DIRECTION_WEAK_LTR;
   context->language = NULL;
   context->font_map = NULL;
 
@@ -325,6 +325,13 @@ pango_context_get_language (PangoContext *context)
  * @direction: the new base direction
  * 
  * Sets the base direction for the context.
+ *
+ * The base direction is used in applying the Unicode bidirectional
+ * algorithm; if the @direction is %PANGO_DIRECTION_LTR or
+ * %PANGO_DIRECTION_RTL, then the value will be used as the paragraph
+ * direction in the Unicode bidirectional algorithm.  A value of
+ * %PANGO_DIRECTION_WEAK_LTR or %PANGO_DIRECTION_WEAK_RTL is used only
+ * for paragraphs that do not contain any strong characters themselves. 
  **/
 void
 pango_context_set_base_dir (PangoContext  *context,
@@ -337,9 +344,10 @@ pango_context_set_base_dir (PangoContext  *context,
 
 /**
  * pango_context_get_base_dir:
- * @context: 
+ * @context: a #PangoContext
  * 
- * Retrieves the base direction for the context.
+ * Retrieves the base direction for the context. See
+ * pango_context_set_base_dir().
  * 
  * Return value: the base direction for the context.
  **/
@@ -507,6 +515,8 @@ struct _ItemizeState
   
   PangoFontset *current_fonts;
   ShaperFontCache *cache;
+  PangoFont *base_font;
+  gboolean enable_fallback;
   
   GSList *exact_engines;
   GSList *fallback_engines;
@@ -526,10 +536,24 @@ update_embedding_end (ItemizeState *state)
   state->changed |= EMBEDDING_CHANGED;
 }
 
+static PangoAttribute *
+find_attribute (GSList        *attr_list,
+                PangoAttrType  type)
+{
+  GSList *node;
+  
+  for (node = attr_list; node; node = node->next)
+    if (((PangoAttribute *) node->data)->klass->type == type)
+      return (PangoAttribute *) node->data;
+
+  return NULL;
+}
+
 static void
 update_attr_iterator (ItemizeState *state)
 {
   PangoLanguage *old_lang;
+  PangoAttribute *fallback;
   int end_index;
   
   pango_attr_iterator_range (state->attr_iter, NULL, &end_index);
@@ -546,6 +570,9 @@ update_attr_iterator (ItemizeState *state)
 				&state->lang, &state->extra_attrs);
   state->copy_extra_attrs = FALSE;
 
+  fallback = find_attribute (state->extra_attrs, PANGO_ATTR_FALLBACK);
+  state->enable_fallback = (fallback == NULL || ((PangoAttrInt *)fallback)->value);
+
   state->changed |= FONT_CHANGED;
   if (state->lang != old_lang)
     state->changed |= LANG_CHANGED;
@@ -565,12 +592,12 @@ static void
 itemize_state_init (ItemizeState      *state,
 		    PangoContext      *context, 
 		    const char        *text,
+		    PangoDirection     base_dir,
 		    int                start_index,
 		    int                length,
 		    PangoAttrList     *attrs,
 		    PangoAttrIterator *cached_iter)
 {
-  PangoDirection base_dir;
   gunichar *text_ucs4;
   long n_chars;
 
@@ -586,8 +613,6 @@ itemize_state_init (ItemizeState      *state,
   /* First, apply the bidirectional algorithm to break
    * the text into directional runs.
    */
-  base_dir = context->base_dir;
-
   text_ucs4 = g_utf8_to_ucs4_fast (text + start_index, length, &n_chars);
   state->embedding_levels = g_new (guint8, n_chars);
   pango_log2vis_get_embedding_levels (text_ucs4, n_chars, &base_dir,
@@ -631,6 +656,7 @@ itemize_state_init (ItemizeState      *state,
   state->cache = NULL;
   state->exact_engines = NULL;
   state->fallback_engines = NULL;
+  state->base_font = NULL;
   
   state->changed = EMBEDDING_CHANGED | SCRIPT_CHANGED | LANG_CHANGED | FONT_CHANGED;
 }
@@ -805,6 +831,16 @@ get_shaper_and_font_foreach (PangoFontset *fontset,
   return FALSE;
 }
 
+static PangoFont *
+get_base_font (ItemizeState *state)
+{
+  if (!state->base_font)
+    state->base_font = pango_font_map_load_font (state->context->font_map,
+                                                 state->context,
+                                                 state->font_desc);
+  return state->base_font;
+}
+
 static gboolean
 get_shaper_and_font (ItemizeState      *state,
 		     gunichar           wc,
@@ -813,7 +849,9 @@ get_shaper_and_font (ItemizeState      *state,
 {
   GetShaperFontInfo info;
 
-  if (shaper_font_cache_get (state->cache, wc, shape_engine, font))
+  /* We'd need a separate cache when fallback is disabled, but since lookup
+   * with fallback disabled is faster anyways, we just skip caching */
+  if (state->enable_fallback && shaper_font_cache_get (state->cache, wc, shape_engine, font))
     return *shape_engine != NULL;
 
   if (!state->exact_engines && !state->fallback_engines)
@@ -826,24 +864,35 @@ get_shaper_and_font (ItemizeState      *state,
   info.font = NULL;
 
   info.engines = state->exact_engines;
-  pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
+  if (state->enable_fallback)
+    pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
+  else 
+    get_shaper_and_font_foreach (NULL, get_base_font (state), &info);
+
   if (info.shape_engine)
     {
       *shape_engine = info.shape_engine;
       *font = info.font;
 
-      shaper_font_cache_insert (state->cache, wc, *shape_engine, *font);
+      /* skip caching if fallback disabled (see above) */
+      if (state->enable_fallback)
+        shaper_font_cache_insert (state->cache, wc, *shape_engine, *font);
 
       return TRUE;
     }
 
   info.engines = state->fallback_engines;
-  pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
+  if (state->enable_fallback)
+    pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
+  else 
+    get_shaper_and_font_foreach (NULL, get_base_font (state), &info);
 
   *shape_engine = info.shape_engine;
   *font = info.font;
 
-  shaper_font_cache_insert (state->cache, wc, *shape_engine, *font);
+  /* skip caching if fallback disabled (see above) */
+  if (state->enable_fallback)
+    shaper_font_cache_insert (state->cache, wc, *shape_engine, *font);
       
   return *shape_engine != NULL;
 }
@@ -937,6 +986,12 @@ itemize_state_update_for_new_run (ItemizeState *state)
 							  state->derived_lang);
       state->cache = get_shaper_font_cache (state->current_fonts);
     }
+
+  if ((state->changed & FONT_CHANGED) && state->base_font)
+    {
+      g_object_unref (state->base_font);
+      state->base_font = NULL;
+    }
 }
 
 static void
@@ -1011,6 +1066,56 @@ itemize_state_finish (ItemizeState *state)
     g_object_unref (state->current_fonts);
 }
 
+/**
+ * pango_itemize_with_base_dir:
+ * @context:   a structure holding information that affects
+               the itemization process.
+ * @text:      the text to itemize.
+ * @start_index: first byte in @text to process
+ * @length:    the number of bytes (not characters) to process
+ *             after @start_index.
+ *             This must be >= 0.
+ * @base_dir:  base direction to use for bidirectional processing
+ * @attrs:     the set of attributes that apply to @text.
+ * @cached_iter:      Cached attribute iterator, or NULL
+ *
+ * Like pango_itemize(), but the base direction to use when
+ * computing bidirectional levels (see pango_context_set_base_dir ()),
+ * is specified explicitely rather than gotten from the #PangoContext.
+ *
+ * Return value: a GList of PangoItem structures.
+ */
+GList *
+pango_itemize_with_base_dir (PangoContext      *context, 
+			     PangoDirection     base_dir,
+			     const char        *text,
+			     int                start_index,
+			     int                length,
+			     PangoAttrList     *attrs,
+			     PangoAttrIterator *cached_iter)
+{
+  ItemizeState state;
+
+  g_return_val_if_fail (context != NULL, NULL);
+  g_return_val_if_fail (start_index >= 0, NULL);
+  g_return_val_if_fail (length >= 0, NULL);
+  g_return_val_if_fail (length == 0 || text != NULL, NULL);
+
+  if (length == 0)
+    return NULL;
+
+  itemize_state_init (&state, context, text, base_dir, start_index, length,
+		      attrs, cached_iter);
+
+  do 
+    itemize_state_process_run (&state);
+  while (itemize_state_next (&state));
+
+  itemize_state_finish (&state);
+  
+  return g_list_reverse (state.result);
+}
+
 /**
  * pango_itemize:
  * @context:   a structure holding information that affects
@@ -1044,26 +1149,13 @@ pango_itemize (PangoContext      *context,
 	       PangoAttrList     *attrs,
                PangoAttrIterator *cached_iter)
 {
-  ItemizeState state;
-
   g_return_val_if_fail (context != NULL, NULL);
   g_return_val_if_fail (start_index >= 0, NULL);
   g_return_val_if_fail (length >= 0, NULL);
   g_return_val_if_fail (length == 0 || text != NULL, NULL);
 
-  if (length == 0)
-    return NULL;
-
-  itemize_state_init (&state, context, text, start_index, length,
-		      attrs, cached_iter);
-
-  do 
-    itemize_state_process_run (&state);
-  while (itemize_state_next (&state));
-
-  itemize_state_finish (&state);
-  
-  return g_list_reverse (state.result);
+  return pango_itemize_with_base_dir (context, context->base_dir,
+				      text, start_index, length, attrs, cached_iter);
 }
 
 /**
