@@ -26,6 +26,7 @@ pango_attr_type_get_type (void)
       { PANGO_ATTR_RISE, "PANGO_ATTR_RISE", "rise" },
       { PANGO_ATTR_SHAPE, "PANGO_ATTR_SHAPE", "shape" },
       { PANGO_ATTR_SCALE, "PANGO_ATTR_SCALE", "scale" },
+      { PANGO_ATTR_FALLBACK, "PANGO_ATTR_FALLBACK", "fallback" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("PangoAttrType", values);
@@ -43,6 +44,7 @@ pango_underline_get_type (void)
       { PANGO_UNDERLINE_SINGLE, "PANGO_UNDERLINE_SINGLE", "single" },
       { PANGO_UNDERLINE_DOUBLE, "PANGO_UNDERLINE_DOUBLE", "double" },
       { PANGO_UNDERLINE_LOW, "PANGO_UNDERLINE_LOW", "low" },
+      { PANGO_UNDERLINE_ERROR, "PANGO_UNDERLINE_ERROR", "error" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("PangoUnderline", values);
@@ -294,6 +296,9 @@ pango_direction_get_type (void)
       { PANGO_DIRECTION_RTL, "PANGO_DIRECTION_RTL", "rtl" },
       { PANGO_DIRECTION_TTB_LTR, "PANGO_DIRECTION_TTB_LTR", "ttb-ltr" },
       { PANGO_DIRECTION_TTB_RTL, "PANGO_DIRECTION_TTB_RTL", "ttb-rtl" },
+      { PANGO_DIRECTION_WEAK_LTR, "PANGO_DIRECTION_WEAK_LTR", "weak-ltr" },
+      { PANGO_DIRECTION_WEAK_RTL, "PANGO_DIRECTION_WEAK_RTL", "weak-rtl" },
+      { PANGO_DIRECTION_NEUTRAL, "PANGO_DIRECTION_NEUTRAL", "neutral" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("PangoDirection", values);
