@@ -55,6 +55,7 @@ struct _PangoLayoutLine
   gint         length;		/* length of line in bytes */
   GSList      *runs;
   guint        is_paragraph_start : 1;  /* TRUE if this is the first line of the paragraph */ 
+  guint        resolved_dir : 3;  /* Resolved direction of line */
 };
 
 #define PANGO_TYPE_LAYOUT              (pango_layout_get_type ())
@@ -111,6 +112,9 @@ int            pango_layout_get_spacing          (PangoLayout                *la
 void           pango_layout_set_justify          (PangoLayout                *layout,
 						  gboolean                    justify);
 gboolean       pango_layout_get_justify          (PangoLayout                *layout);
+void           pango_layout_set_auto_dir          (PangoLayout                *layout,
+						  gboolean                    auto_dir);
+gboolean       pango_layout_get_auto_dir          (PangoLayout                *layout);
 void           pango_layout_set_alignment        (PangoLayout                *layout,
 						  PangoAlignment              alignment);
 PangoAlignment pango_layout_get_alignment        (PangoLayout                *layout);
