@@ -1177,8 +1177,8 @@ static LangInfo lang_texts[] = {
   { "fi", "Finnish (Suomi)  Hei, Hyv\303\244\303\244 p\303\244iv\303\244\303\244" },
   { "fr", "French (Fran\303\247ais)" },
   { "de", "German Gr\303\274\303\237 Gott" },
-  { "iw", "Hebrew   \327\251\327\234\327\225\327\235" },
-  { "il", "Italiano  Ciao, Buon giorno" },
+  { "he", "Hebrew   \327\251\327\234\327\225\327\235" },
+  { "it", "Italiano  Ciao, Buon giorno" },
   { "ja", "Japanese (\346\227\245\346\234\254\350\252\236) \343\201\223\343\202\223\343\201\253\343\201\241\343\201\257, \357\275\272\357\276\235\357\276\206\357\276\201\357\276\212" },
   { "ko", "Korean (\355\225\234\352\270\200)   \354\225\210\353\205\225\355\225\230\354\204\270\354\232\224, \354\225\210\353\205\225\355\225\230\354\213\255\353\213\210\352\271\214" },
   { "mt", "Maltese   \304\212aw, Sa\304\247\304\247a" },
@@ -1253,16 +1253,15 @@ pango_log2vis_get_embedding_levels (gunichar       *str,
   return result;
 }
 
+#endif /* HAVE_FRIBIDI */
 
 gboolean 
 pango_get_mirror_char (gunichar        ch,
 		       gunichar       *mirrored_ch)
 {
-  return fribidi_get_mirror_char (ch, mirrored_ch); 
+  return g_unichar_get_mirror_char (ch, mirrored_ch); 
 }
 
-#endif /* HAVE_FRIBIDI */
-
 
 static guint
 alias_hash (struct PangoAlias *alias)
@@ -1494,3 +1493,39 @@ pango_lookup_aliases (const char   *fontname,
       *n_families = 0;
     }
 }
+
+/**
+ * pango_find_base_dir:
+ * @text:   the text to process
+ * @length: length of @text in bytes (may be -1 if @text is nul-terminated)
+ *
+ * Searches a string the first character that has a strong 
+ * direction, according to the Unicode bidirectional algorithm.
+ *
+ * Return value: The direction corresponding to the first strong character.
+ * If no such character is found, then %PANGO_DIRECTION_NEUTRAL is returned.
+ */
+PangoDirection
+pango_find_base_dir (const gchar *text,
+		     gint         length)
+{
+  PangoDirection dir = PANGO_DIRECTION_NEUTRAL;
+  const gchar *p;
+
+  g_return_val_if_fail (text != NULL, PANGO_DIRECTION_NEUTRAL);
+
+  p = text;
+  while ((length < 0 || p < text + length) && *p)
+    {
+      gunichar wc = g_utf8_get_char (p);
+
+      dir = pango_unichar_direction (wc);
+
+      if (dir != PANGO_DIRECTION_NEUTRAL)
+	break;
+
+      p = g_utf8_next_char (p);
+    }
+
+  return dir;
+}
